/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AuditService.java 19425 2010-03-12 11:53:42Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.service;

import java.io.Serializable;
import java.util.ArrayList;

import javax.management.MBeanServerConnection;
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import org.ow2.jonas.lib.management.extensions.manager.ManagementEntryPoint;
import org.ow2.util.auditreport.api.AuditorJMXObjectNames;
import org.ow2.util.auditreport.impl.util.ObjectEncoder;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Listener for audit notifications.
 */
public class AuditService implements Serializable, NotificationListener {
    /**
     * Logger of the class
     */
    private Log logger = LogFactory.getLog(AuditService.class);
    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = 1L;
    /**
     * Reports waiting to be displayed.
     */
    private ArrayList reportsObjects = new ArrayList();
    /**
     * The EZB JMX server.
     */
    private transient MBeanServerConnection serv = null;
    /**
     * Mbean name of EZB auditor.
     */
    private ObjectName ejbNotifier = null;
    /**
     * Mbean name of web auditor.
     */
    private ObjectName webNotifier = null;
     /**
     * Mbean name of jndi auditor.
     */
    private ObjectName jndiNotifier = null;

    /**
     * Number of invocations detected.
     */
    private int invocations = 0;

    /**
     * Domain name.
     */
    private String domainName = null;
    /**
     * Constructor.
     */
    public AuditService() {
        init();
    }

    /**
     * Init the notifications listening.
     */
    private void init() {
        reportsObjects = new ArrayList();
        try {
            domainName = ManagementEntryPoint.getInstance().getDomainName();
            serv = ManagementEntryPoint.getInstance().getServerConnection();
            ejbNotifier = new ObjectName(domainName
                    + AuditorJMXObjectNames.EJBAUDITOR_TYPE_COMPONENT + ",name=EasyBeans");
            webNotifier = new ObjectName(domainName
                    + AuditorJMXObjectNames.WEBAUDITOR_TYPE_COMPONENT + ",name=JOnAS");
            jndiNotifier = new ObjectName(domainName
                    + AuditorJMXObjectNames.JNDIAUDITOR_TYPE_COMPONENT + ",name=JOnAS");
        } catch (Exception ex) {
            logger.error("Error when creating notifs listeners : ", ex);
        }
    }

    /**
     * Start the listening.
     */
    public void startListening() {
        if(serv == null) {
            init();
        }
        getNewRegistrations();
        this.reportsObjects.clear();
    }
    /**
     * @return waiting reports.
     */
    public ArrayList getReportObjects() {

        ArrayList tmp = (ArrayList) this.reportsObjects.clone();
        this.reportsObjects.clear();
        return tmp;
    }
    /**
     * Start the audit on every auditor component.
     */
    public void startAuditing() {
        // TODO : start jonas auditing services .
    }
    /**
     * Stop the audit on every auditor component.
     */
    public void stopAuditing() {
       // TODO : stop jonas auditing services.
    }
    /**
     * This methode handle a JMX notification.
     * @param notification the notification.
     * @param handback the handback.
     */
    public void handleNotification(final Notification notification, final Object handback) {
        this.reportsObjects.add(ObjectEncoder.decode(notification.getMessage()));
        invocations++;
    }
    /**
     * This method add a listener on a MBean if this MBean is present in
     * the JMX server.
     */
    public void getNewRegistrations() {
        logger.info("Start listening audit notifications");
        try {
            if (serv != null) {
                if (serv.isRegistered(ejbNotifier)) {
                    serv.addNotificationListener(ejbNotifier, this, null, null);
                }
                if (serv.isRegistered(webNotifier)) {
                    serv.addNotificationListener(webNotifier, this, null, null);
                }
                if (serv.isRegistered(jndiNotifier)) {
                    serv.addNotificationListener(jndiNotifier, this, null, null);
                }
            }
        } catch (Exception ex) {
            logger.error("Error at listeners registration : ", ex);
        }
    }
    /**
     * Unregister the JMX listener.
     */
    public void unregister() {
        logger.info("Stop listening audit notifications");
        try {
            if (serv.isRegistered(ejbNotifier)) {
                serv.removeNotificationListener(ejbNotifier, this);
            }
            if (serv.isRegistered(webNotifier)) {
                serv.removeNotificationListener(webNotifier, this);
            }
            if (serv.isRegistered(jndiNotifier)) {
                serv.removeNotificationListener(jndiNotifier, this);
            }
        } catch (Exception ex) {
           logger.error("Error at listeners unregistration : ", ex);
        }
    }
    /**
     * @return number of invocations.
     */
    public int getInvocations() {
        return invocations;
    }
}
